@@ -1,20 +1,21 @@
-/* Copyright (C) 1998,1999,2000,2001,2002,2003, 2006, 2008, 2009, 2010 Free Software Foundation, Inc.
- * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1998-2003,2006,2008-2013,2015,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -22,13 +23,16 @@
 # include <config.h>
 #endif
 
-#include "libguile/_scm.h"
-#include "libguile/eval.h"
-#include "libguile/fluids.h"
-#include "libguile/modules.h"
+#include "gsubr.h"
+#include "eval.h"
+#include "list.h"
+#include "fluids.h"
+#include "modules.h"
+#include "pairs.h"
+#include "symbols.h"
+#include "variable.h"
 
-#include "libguile/validate.h"
-#include "libguile/evalext.h"
+#include "evalext.h"
 
 SCM_DEFINE (scm_defined_p, "defined?", 1, 1, 0,
             (SCM sym, SCM module),
@@ -55,9 +59,6 @@ SCM_DEFINE (scm_defined_p, "defined?", 1, 1, 0,
 #undef FUNC_NAME
 
 
-SCM_REGISTER_PROC (s_map_in_order, "map-in-order", 2, 0, 1, scm_map);
-
-
 SCM_DEFINE (scm_self_evaluating_p, "self-evaluating?", 1, 0, 0,
 	    (SCM obj),
 	    "Return #t for objects which Guile considers self-evaluating")
@@ -79,17 +80,21 @@ SCM_DEFINE (scm_self_evaluating_p, "self-evaluating?", 1, 0, 0,
 	case scm_tc7_wvect:
 	case scm_tc7_pointer:
 	case scm_tc7_hashtable:
+	case scm_tc7_weak_set:
+	case scm_tc7_weak_table:
 	case scm_tc7_fluid:
 	case scm_tc7_dynamic_state:
         case scm_tc7_frame:
-        case scm_tc7_objcode:
-        case scm_tc7_vm:
+        case scm_tc7_keyword:
+        case scm_tc7_syntax:
         case scm_tc7_vm_cont:
 	case scm_tc7_number:
 	case scm_tc7_string:
 	case scm_tc7_smob:
 	case scm_tc7_program:
 	case scm_tc7_bytevector:
+	case scm_tc7_array:
+	case scm_tc7_bitvector:
 	case scm_tcs_struct:
 	  return SCM_BOOL_T;
 	default:
@@ -105,11 +110,5 @@ SCM_DEFINE (scm_self_evaluating_p, "self-evaluating?", 1, 0, 0,
 void 
 scm_init_evalext ()
 {
-#include "libguile/evalext.x"
+#include "evalext.x"
 }
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
