@@ -1,39 +1,41 @@
-/* Copyright (C) 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
- *   2006, 2007, 2009, 2011, 2013 Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1995-1996,1998-2004,2006-2007,2009,2011,2013,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 
-#include "libguile/_scm.h"
-#include "libguile/root.h"
-#include "libguile/strings.h"
-#include "libguile/validate.h"
-#include "libguile/fluids.h"
+#include "fluids.h"
+#include "gsubr.h"
+#include "modules.h"
+#include "pairs.h"
+#include "strings.h"
+#include "symbols.h"
+#include "variable.h"
+
+#include "feature.h"
 
-#include "libguile/feature.h"
 
 
 
@@ -107,9 +109,6 @@ scm_init_feature()
 #ifndef _Windows
   scm_add_feature("system");
 #endif
-#ifdef vms
-  scm_add_feature(s_ed);
-#endif
 #ifndef GO32
   scm_add_feature("char-ready?");
 #endif
@@ -117,11 +116,5 @@ scm_init_feature()
   scm_add_feature ("threads");
 #endif
 
-#include "libguile/feature.x"
+#include "feature.x"
 }
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
