@@ -1,67 +1,102 @@
-/* classes: h_files */
-
 #ifndef SCM_NUMBERS_H
 #define SCM_NUMBERS_H
 
-/* Copyright (C) 1995,1996,1998,2000,2001,2002,2003,2004,2005, 2006,
- *   2008, 2009, 2010, 2011, 2013 Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1995-1996,1998,2000-2006,2008-2011,2013-2014,2016-2018,2021-2022
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
+#include "libguile/scmconfig.h"
+
+/* gmp.h needs to be included with C++ linkage, if including Guile
+   headers from a C++ compiler.  */
+#ifdef __cplusplus
+extern "C++" {
+#endif
+
+#if SCM_ENABLE_MINI_GMP
+#ifdef BUILDING_LIBGUILE
+#include "libguile/mini-gmp.h"
+#endif
+#else
 #include <gmp.h>
+#endif
 
-#include "libguile/__scm.h"
+#ifdef __cplusplus
+}
+#endif
+
+#include "libguile/error.h"
+#include "libguile/gc.h"
 #include "libguile/print.h"
 
-#ifndef SCM_T_WCHAR_DEFINED
-typedef scm_t_int32 scm_t_wchar;
-#define SCM_T_WCHAR_DEFINED
-#endif /* SCM_T_WCHAR_DEFINED */
-
 
 
 /* Immediate Numbers, also known as fixnums
  *
- * Inums are exact integer data that fits within an SCM word.  */
-
-/* SCM_T_SIGNED_MAX is                 (- (expt 2 n) 1),
- * SCM_MOST_POSITIVE_FIXNUM should be  (- (expt 2 (- n 2)) 1)
- * which is the same as                (/ (- (expt 2 n) 4) 4)
- */
-
+ * Inums are exact integers that fit within an SCM word
+ * (along with two tagging bits).
+ *
+ * In the current implementation, Inums must also fit within a long
+ * because that's what GMP's mpz_*_si functions accept.  */
+typedef long scm_t_inum;
 #define SCM_I_FIXNUM_BIT         (SCM_LONG_BIT - 2)
-#define SCM_MOST_POSITIVE_FIXNUM ((SCM_T_SIGNED_BITS_MAX-3)/4)
-#define SCM_MOST_NEGATIVE_FIXNUM (-SCM_MOST_POSITIVE_FIXNUM-1)
+#define SCM_MOST_NEGATIVE_FIXNUM (-1L << (SCM_I_FIXNUM_BIT - 1))
+#define SCM_MOST_POSITIVE_FIXNUM (- (SCM_MOST_NEGATIVE_FIXNUM + 1))
 
-/* SCM_SRS is signed right shift */
-#if (-1 == (((-1) << 2) + 2) >> 2)
-# define SCM_SRS(x, y) ((x) >> (y))
+/* SCM_SRS (X, Y) is signed right shift, defined as floor (X / 2^Y),
+   where Y must be non-negative and less than the width in bits of X.
+   It's common for >> to do this, but the C standards do not specify
+   what happens when X is negative.
+
+   NOTE: X must not perform side effects.  */
+#if (-1 >> 2 == -1) && (-4 >> 2 == -1) && (-5 >> 2 == -2) && (-8 >> 2 == -2)
+# define SCM_SRS(x, y)  ((x) >> (y))
 #else
-# define SCM_SRS(x, y) ((x) < 0 ? ~((~(x)) >> (y)) : ((x) >> (y)))
-#endif /* (-1 == (((-1) << 2) + 2) >> 2) */
+# define SCM_SRS(x, y)                                   \
+  ((x) < 0                                               \
+   ? -1 - (scm_t_signed_bits) (~(scm_t_bits)(x) >> (y))  \
+   : ((x) >> (y)))
+#endif
 
 
+/* The first implementation of SCM_I_INUM below depends on behavior that
+   is specified by GNU C but not by C standards, namely that when
+   casting to a signed integer of width N, the value is reduced modulo
+   2^N to be within range of the type.  The second implementation below
+   should be portable to all conforming C implementations, but may be
+   less efficient if the compiler is not sufficiently clever.
+
+   NOTE: X must not perform side effects.  */
+#ifdef __GNUC__
+# define SCM_I_INUM(x)  (SCM_SRS ((scm_t_inum) SCM_UNPACK (x), 2))
+#else
+# define SCM_I_INUM(x)                          \
+  (SCM_UNPACK (x) > SCM_T_SIGNED_BITS_MAX       \
+   ? -1 - (scm_t_inum) (~SCM_UNPACK (x) >> 2)   \
+   : (scm_t_inum) (SCM_UNPACK (x) >> 2))
+#endif
+
 #define SCM_I_INUMP(x)	(2 & SCM_UNPACK (x))
 #define SCM_I_NINUMP(x) (!SCM_I_INUMP (x))
 #define SCM_I_MAKINUM(x) \
-  (SCM_PACK ((((scm_t_signed_bits) (x)) << 2) + scm_tc2_int))
-#define SCM_I_INUM(x)   (SCM_SRS ((scm_t_signed_bits) SCM_UNPACK (x), 2))
+  (SCM_PACK ((((scm_t_bits) (x)) << 2) + scm_tc2_int))
 
 /* SCM_FIXABLE is true if its long argument can be encoded in an SCM_INUM. */
 #define SCM_POSFIXABLE(n) ((n) <= SCM_MOST_POSITIVE_FIXNUM)
@@ -77,15 +112,13 @@ typedef scm_t_int32 scm_t_wchar;
  * SCM_FLTMAX is less than or scm_equal the largest single precision float
  */
 
-#if SCM_HAVE_STDC_HEADERS
-# ifndef GO32
-#  include <float.h>
-#  ifdef __MINGW32__
-#   define copysign _copysign
-#   define finite _finite
-#  endif /* __MINGW32__ */
-# endif /* ndef GO32 */
-#endif /* def STDC_HEADERS */
+#ifndef GO32
+# include <float.h>
+# ifdef __MINGW32__
+#  define copysign _copysign
+#  define finite _finite
+# endif /* __MINGW32__ */
+#endif /* ndef GO32 */
 
 #ifdef DBL_MAX_10_EXP
 # define SCM_MAXEXP DBL_MAX_10_EXP
@@ -101,11 +134,11 @@ typedef scm_t_int32 scm_t_wchar;
 
 
 /* SCM_INTBUFLEN is the maximum number of characters neccessary for
- * the printed or scm_string representation of an scm_t_intmax in
+ * the printed or scm_string representation of an intmax_t in
  * radix 2.  The buffer passed to scm_iint2str and scm_iuint2str must
  * be of this size, for example.
  */
-#define SCM_INTBUFLEN (5 + SCM_CHAR_BIT*sizeof(scm_t_intmax))
+#define SCM_INTBUFLEN (5 + SCM_CHAR_BIT*sizeof(intmax_t))
 
 
 
@@ -117,7 +150,7 @@ typedef scm_t_int32 scm_t_wchar;
  * differ in one bit: This way, checking if an object is an inexact number can
  * be done quickly (using the TYP16S macro).  */
 
-/* Number subtype 1 to 3 (note the dependency on the predicates SCM_INEXACTP
+/* Number subtype 1 to 4 (note the dependency on the predicates SCM_INEXACTP
  * and SCM_NUMP)  */
 #define scm_tc16_big		(scm_tc7_number + 1 * 256L)
 #define scm_tc16_real           (scm_tc7_number + 2 * 256L)
@@ -126,22 +159,19 @@ typedef scm_t_int32 scm_t_wchar;
 
 #define SCM_INEXACTP(x) \
   (!SCM_IMP (x) && (0xfeff & SCM_CELL_TYPE (x)) == scm_tc16_real)
-#define SCM_REALP(x) (!SCM_IMP (x) && SCM_TYP16 (x) == scm_tc16_real)
-#define SCM_COMPLEXP(x) (!SCM_IMP (x) && SCM_TYP16 (x) == scm_tc16_complex)
+#define SCM_REALP(x) (SCM_HAS_TYP16 (x, scm_tc16_real))
+#define SCM_COMPLEXP(x) (SCM_HAS_TYP16 (x, scm_tc16_complex))
 
 #define SCM_REAL_VALUE(x) (((scm_t_double *) SCM2PTR (x))->real)
 #define SCM_COMPLEX_REAL(x) (((scm_t_complex *) SCM2PTR (x))->real)
 #define SCM_COMPLEX_IMAG(x) (((scm_t_complex *) SCM2PTR (x))->imag)
 
-/* Each bignum is just an mpz_t stored in a double cell starting at word 1. */
-#define SCM_I_BIG_MPZ(x) (*((mpz_t *) (SCM_CELL_OBJECT_LOC((x),1))))
-#define SCM_BIGP(x) (!SCM_IMP (x) && SCM_TYP16 (x) == scm_tc16_big)
+#define SCM_BIGP(x) (SCM_HAS_TYP16 (x, scm_tc16_big))
 
 #define SCM_NUMBERP(x) (SCM_I_INUMP(x) || SCM_NUMP(x))
-#define SCM_NUMP(x) (!SCM_IMP(x) \
-		     && ((0x00ff & SCM_CELL_TYPE (x)) == scm_tc7_number))
+#define SCM_NUMP(x) (SCM_HAS_TYP7 (x, scm_tc7_number))
 
-#define SCM_FRACTIONP(x) (!SCM_IMP (x) && SCM_TYP16 (x) == scm_tc16_fraction)
+#define SCM_FRACTIONP(x) (SCM_HAS_TYP16 (x, scm_tc16_fraction))
 #define SCM_FRACTION_NUMERATOR(x) (SCM_CELL_OBJECT_1 (x))
 #define SCM_FRACTION_DENOMINATOR(x) (SCM_CELL_OBJECT_2 (x))
 
@@ -150,19 +180,22 @@ typedef scm_t_int32 scm_t_wchar;
 typedef struct scm_t_double
 {
   SCM type;
+#if SCM_SIZEOF_UINTPTR_T != 8
   SCM pad;
+#endif
   double real;
 } scm_t_double;
 
 typedef struct scm_t_complex
 {
   SCM type;
+#if SCM_SIZEOF_UINTPTR_T != 8
   SCM pad;
+#endif
   double real;
   double imag;
 } scm_t_complex;
 
-
 
 
 SCM_API SCM scm_exact_p (SCM x);
@@ -225,8 +258,8 @@ SCM_INTERNAL SCM scm_i_logand (SCM x, SCM y, SCM rest);
 SCM_INTERNAL SCM scm_i_logior (SCM x, SCM y, SCM rest);
 SCM_INTERNAL SCM scm_i_logxor (SCM x, SCM y, SCM rest);
 
-SCM_API size_t scm_iint2str (scm_t_intmax num, int rad, char *p);
-SCM_API size_t scm_iuint2str (scm_t_uintmax num, int rad, char *p);
+SCM_API size_t scm_iint2str (intmax_t num, int rad, char *p);
+SCM_API size_t scm_iuint2str (uintmax_t num, int rad, char *p);
 SCM_API SCM scm_number_to_string (SCM x, SCM radix);
 SCM_API int scm_print_real (SCM sexp, SCM port, scm_print_state *pstate);
 SCM_API int scm_print_complex (SCM sexp, SCM port, scm_print_state *pstate);
@@ -238,11 +271,13 @@ SCM_API SCM scm_string_to_number (SCM str, SCM radix);
 SCM_API SCM scm_bigequal (SCM x, SCM y);
 SCM_API SCM scm_real_equalp (SCM x, SCM y);
 SCM_API SCM scm_complex_equalp (SCM x, SCM y);
+SCM_INTERNAL int scm_i_heap_numbers_equal_p (SCM x, SCM y);
 SCM_API SCM scm_number_p (SCM x);
 SCM_API SCM scm_complex_p (SCM x);
 SCM_API SCM scm_real_p (SCM x);
 SCM_API SCM scm_rational_p (SCM z);
 SCM_API SCM scm_integer_p (SCM x);
+SCM_API SCM scm_exact_integer_p (SCM x);
 SCM_API SCM scm_inexact_p (SCM x);
 SCM_API int scm_is_inexact (SCM x);
 SCM_API SCM scm_num_eq_p (SCM x, SCM y);
@@ -303,17 +338,6 @@ SCM_INTERNAL SCM scm_i_product (SCM x, SCM y, SCM rest);
 SCM_INTERNAL SCM scm_i_divide (SCM x, SCM y, SCM rest);
 SCM_INTERNAL SCM scm_i_exact_integer_sqrt (SCM k);
 
-/* bignum internal functions */
-SCM_INTERNAL SCM scm_i_mkbig (void);
-SCM_API /* FIXME: not internal */ SCM scm_i_normbig (SCM x);
-SCM_INTERNAL int scm_i_bigcmp (SCM a, SCM b);
-SCM_INTERNAL SCM scm_i_dbl2big (double d);
-SCM_INTERNAL SCM scm_i_dbl2num (double d);
-SCM_API /* FIXME: not internal */ double scm_i_big2dbl (SCM b);
-SCM_API /* FIXME: not internal */ SCM scm_i_long2big (long n);
-SCM_API /* FIXME: not internal */ SCM scm_i_ulong2big (unsigned long n);
-SCM_API /* FIXME: not internal */ SCM scm_i_clonebig (SCM src_big, int same_sign_p);
-
 /* ratio functions */
 SCM_API SCM scm_rationalize (SCM x, SCM err);
 SCM_API SCM scm_numerator (SCM z);
@@ -331,50 +355,56 @@ SCM_INTERNAL void scm_i_print_complex (double real, double imag, SCM port);
 /* conversion functions for integers */
 
 SCM_API int scm_is_integer (SCM val);
+SCM_API int scm_is_exact_integer (SCM val);
 SCM_API int scm_is_signed_integer (SCM val,
-				   scm_t_intmax min, scm_t_intmax max);
+				   intmax_t min, intmax_t max);
 SCM_API int scm_is_unsigned_integer (SCM val,
-				     scm_t_uintmax min, scm_t_uintmax max);
+				     uintmax_t min, uintmax_t max);
 
-SCM_API SCM scm_from_signed_integer (scm_t_intmax val);
-SCM_API SCM scm_from_unsigned_integer (scm_t_uintmax val);
+SCM_API SCM scm_from_signed_integer (intmax_t val);
+SCM_API SCM scm_from_unsigned_integer (uintmax_t val);
 
-SCM_API scm_t_intmax scm_to_signed_integer (SCM val,
-					    scm_t_intmax min,
-					    scm_t_intmax max);
-SCM_API scm_t_uintmax scm_to_unsigned_integer (SCM val,
-					       scm_t_uintmax min,
-					       scm_t_uintmax max);
+SCM_API intmax_t scm_to_signed_integer (SCM val,
+                                        intmax_t min,
+                                        intmax_t max);
+SCM_API uintmax_t scm_to_unsigned_integer (SCM val,
+                                           uintmax_t min,
+                                           uintmax_t max);
 
-SCM_API scm_t_int8   scm_to_int8     (SCM x);
-SCM_API SCM          scm_from_int8   (scm_t_int8 x);
+SCM_API int8_t       scm_to_int8     (SCM x);
+SCM_API SCM          scm_from_int8   (int8_t x);
 
-SCM_API scm_t_uint8  scm_to_uint8    (SCM x);
-SCM_API SCM          scm_from_uint8  (scm_t_uint8 x);
+SCM_API uint8_t      scm_to_uint8    (SCM x);
+SCM_API SCM          scm_from_uint8  (uint8_t x);
 
-SCM_API scm_t_int16  scm_to_int16    (SCM x);
-SCM_API SCM          scm_from_int16  (scm_t_int16 x);
+SCM_API int16_t      scm_to_int16    (SCM x);
+SCM_API SCM          scm_from_int16  (int16_t x);
 
-SCM_API scm_t_uint16 scm_to_uint16   (SCM x);
-SCM_API SCM          scm_from_uint16 (scm_t_uint16 x);
+SCM_API uint16_t     scm_to_uint16   (SCM x);
+SCM_API SCM          scm_from_uint16 (uint16_t x);
 
-SCM_API scm_t_int32  scm_to_int32    (SCM x);
-SCM_API SCM          scm_from_int32  (scm_t_int32 x);
+SCM_API int32_t      scm_to_int32    (SCM x);
+SCM_API SCM          scm_from_int32  (int32_t x);
 
-SCM_API scm_t_uint32 scm_to_uint32   (SCM x);
-SCM_API SCM          scm_from_uint32 (scm_t_uint32 x);
+SCM_API uint32_t     scm_to_uint32   (SCM x);
+SCM_API SCM          scm_from_uint32 (uint32_t x);
 
 SCM_API scm_t_wchar  scm_to_wchar    (SCM x);
 SCM_API SCM          scm_from_wchar  (scm_t_wchar x);
 
-SCM_API scm_t_int64  scm_to_int64    (SCM x);
-SCM_API SCM          scm_from_int64  (scm_t_int64 x);
+SCM_API int64_t      scm_to_int64    (SCM x);
+SCM_API SCM          scm_from_int64  (int64_t x);
 
-SCM_API scm_t_uint64 scm_to_uint64   (SCM x);
-SCM_API SCM          scm_from_uint64 (scm_t_uint64 x);
+SCM_API uint64_t     scm_to_uint64   (SCM x);
+SCM_API SCM          scm_from_uint64 (uint64_t x);
 
+#if defined BUILDING_LIBGUILE && SCM_ENABLE_MINI_GMP
+SCM_INTERNAL void scm_to_mpz (SCM x, mpz_t rop);
+SCM_INTERNAL SCM  scm_from_mpz (mpz_t rop);
+#elif !SCM_ENABLE_MINI_GMP
 SCM_API void scm_to_mpz (SCM x, mpz_t rop);
 SCM_API SCM  scm_from_mpz (mpz_t rop);
+#endif
 
 
 /* The conversion functions for other types are aliased to the
@@ -471,7 +501,7 @@ SCM_API SCM  scm_from_mpz (mpz_t rop);
 #define scm_to_uintmax   scm_to_uint64
 #define scm_from_uintmax scm_from_uint64
 #else
-#error sizeof(scm_t_intmax) is not 4 or 8.
+#error sizeof(intmax_t) is not 4 or 8.
 #endif
 #endif
 
@@ -511,10 +541,38 @@ SCM_API SCM  scm_from_mpz (mpz_t rop);
 #define scm_to_ptrdiff_t    scm_to_int64
 #define scm_from_ptrdiff_t  scm_from_int64
 #else
-#error sizeof(scm_t_ptrdiff) is not 4 or 8.
+#error sizeof(ptrdiff_t) is not 4 or 8.
 #endif
 #endif
 
+#if SCM_SIZEOF_INTPTR_T == 0
+/* No intptr_t; use size_t functions. */
+#define scm_to_intptr_t   scm_to_ssize_t
+#define scm_from_intptr_t scm_from_ssize_t
+#elif SCM_SIZEOF_INTPTR_T == 4
+#define scm_to_intptr_t   scm_to_int32
+#define scm_from_intptr_t scm_from_int32
+#elif SCM_SIZEOF_INTPTR_T == 8
+#define scm_to_intptr_t   scm_to_int64
+#define scm_from_intptr_t scm_from_int64
+#else
+#error sizeof(intptr_t) is not 4 or 8.
+#endif
+
+#if SCM_SIZEOF_UINTPTR_T == 0
+/* No uintptr_t; use size_t functions. */
+#define scm_to_uintptr_t   scm_to_size_t
+#define scm_from_uintptr_t scm_from_size_t
+#elif SCM_SIZEOF_UINTPTR_T == 4
+#define scm_to_uintptr_t   scm_to_uint32
+#define scm_from_uintptr_t scm_from_uint32
+#elif SCM_SIZEOF_UINTPTR_T == 8
+#define scm_to_uintptr_t   scm_to_uint64
+#define scm_from_uintptr_t scm_from_uint64
+#else
+#error sizeof(uintptr_t) is not 4 or 8.
+#endif
+
 /* conversion functions for double */
 
 SCM_API int scm_is_real (SCM val);
@@ -534,15 +592,137 @@ SCM_API double scm_c_angle (SCM z);
 
 SCM_API int scm_is_number (SCM val);
 
-/* If nonzero, tell gmp to use GC_malloc for its allocations.  */
-SCM_API int scm_install_gmp_memory_functions;
-
 SCM_INTERNAL void scm_init_numbers (void);
 
-#endif  /* SCM_NUMBERS_H */
+
 
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
+#define SCM_NUM2SIZE(pos, arg) (scm_to_size_t (arg))
+
+#define SCM_NUM2SIZE_DEF(pos, arg, def) \
+  (SCM_UNBNDP (arg) ? def : scm_to_size_t (arg))
+
+#define SCM_NUM2PTRDIFF(pos, arg) (scm_to_ssize_t (arg))
+
+#define SCM_NUM2PTRDIFF_DEF(pos, arg, def) \
+  (SCM_UNBNDP (arg) ? def : scm_to_ssize_t (arg))
+
+#define SCM_NUM2SHORT(pos, arg) (scm_to_short (arg))
+
+#define SCM_NUM2SHORT_DEF(pos, arg, def) \
+  (SCM_UNBNDP (arg) ? def : scm_to_short (arg))
+
+#define SCM_NUM2USHORT(pos, arg) (scm_to_ushort (arg))
+
+#define SCM_NUM2USHORT_DEF(pos, arg, def) \
+  (SCM_UNBNDP (arg) ? def : scm_to_ushort (arg))
+
+#define SCM_NUM2INT(pos, arg) (scm_to_int (arg))
+
+#define SCM_NUM2INT_DEF(pos, arg, def) \
+  (SCM_UNBNDP (arg) ? def : scm_to_int (arg))
+
+#define SCM_NUM2UINT(pos, arg) (scm_to_uint (arg))
+
+#define SCM_NUM2UINT_DEF(pos, arg, def) \
+  (SCM_UNBNDP (arg) ? def : scm_to_uint (arg))
+
+#define SCM_NUM2ULONG(pos, arg) (scm_to_ulong (arg))
+
+#define SCM_NUM2ULONG_DEF(pos, arg, def) \
+  (SCM_UNBNDP (arg) ? def : scm_to_ulong (arg))
+
+#define SCM_NUM2LONG(pos, arg) (scm_to_long (arg))
+
+#define SCM_NUM2LONG_DEF(pos, arg, def) \
+  (SCM_UNBNDP (arg) ? def : scm_to_long (arg))
+
+#define SCM_NUM2LONG_LONG(pos, arg) (scm_to_long_long (arg))
+
+#define SCM_NUM2LONG_LONG_DEF(pos, arg, def) \
+  (SCM_UNBNDP (arg) ? def : scm_to_long_long (arg))
+
+#define SCM_NUM2ULONG_LONG(pos, arg) (scm_to_ulong_long (arg))
+
+#define SCM_NUM2ULONG_LONG_DEF(pos, arg, def) \
+  (SCM_UNBNDP (arg) ? def : scm_to_ulong_long (arg))
+
+#define SCM_NUM2SIZE(pos, arg) (scm_to_size_t (arg))
+
+#define SCM_NUM2FLOAT(pos, arg) ((float) scm_to_double (arg))
+
+#define SCM_NUM2DOUBLE(pos, arg) (scm_to_double (arg))
+
+#define SCM_OUT_OF_RANGE(pos, arg) \
+  do { scm_out_of_range_pos (FUNC_NAME, arg, scm_from_int (pos)); } while (0)
+
+#define SCM_ASSERT_RANGE(pos, arg, f)					\
+  do { if (SCM_UNLIKELY (!(f)))					\
+         scm_out_of_range_pos (FUNC_NAME, arg, scm_from_int (pos)); }	\
+  while (0)
+
+#define SCM_VALIDATE_REAL(pos, z) SCM_MAKE_VALIDATE_MSG (pos, z, REALP, "real")
+
+#define SCM_VALIDATE_NUMBER(pos, z) SCM_MAKE_VALIDATE_MSG (pos, z, NUMBERP, "number")
+
+#define SCM_VALIDATE_USHORT_COPY(pos, k, cvar) \
+  do { \
+    cvar = SCM_NUM2USHORT (pos, k); \
+  } while (0)
+
+#define SCM_VALIDATE_SHORT_COPY(pos, k, cvar) \
+  do { \
+    cvar = SCM_NUM2SHORT (pos, k); \
+  } while (0)
+
+#define SCM_VALIDATE_UINT_COPY(pos, k, cvar) \
+  do { \
+    cvar = SCM_NUM2UINT (pos, k); \
+  } while (0)
+
+#define SCM_VALIDATE_INT_COPY(pos, k, cvar) \
+  do { \
+    cvar = SCM_NUM2INT (pos, k); \
+  } while (0)
+
+#define SCM_VALIDATE_ULONG_COPY(pos, k, cvar) \
+  do { \
+    cvar = SCM_NUM2ULONG (pos, k); \
+  } while (0)
+
+#define SCM_VALIDATE_LONG_COPY(pos, k, cvar) \
+  do { \
+    cvar = SCM_NUM2LONG (pos, k); \
+  } while (0)
+
+#define SCM_VALIDATE_SIZE_COPY(pos, k, cvar) \
+  do { \
+    cvar = SCM_NUM2SIZE (pos, k);              \
+  } while (0)
+
+#define SCM_VALIDATE_FLOAT_COPY(pos, k, cvar) \
+  do { \
+    cvar = SCM_NUM2FLOAT (pos, k); \
+  } while (0)
+
+#define SCM_VALIDATE_DOUBLE_COPY(pos, k, cvar) \
+  do { \
+    cvar = SCM_NUM2DOUBLE (pos, k); \
+  } while (0)
+
+#define SCM_VALIDATE_DOUBLE_DEF_COPY(pos, k, default, cvar) \
+  do { \
+    if (SCM_UNBNDP (k)) \
+      { \
+        k = scm_make_real (default); \
+        cvar = default; \
+      } \
+    else \
+      { \
+        cvar = SCM_NUM2DOUBLE (pos, k); \
+      } \
+  } while (0)
+
+
+
+
+#endif  /* SCM_NUMBERS_H */
