@@ -1,6 +1,7 @@
 ;;; Parsing Guile's command-line
 
-;;; Copyright (C) 1994-1998, 2000-2011, 2012, 2013 Free Software Foundation, Inc.
+;;; Copyright (C) 1994-1998, 2000-2024 Free Software Foundation, Inc.
+;;; Copyright (C) 2025 Noé Lopez <noelopez@free.fr>
 
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -32,7 +33,7 @@
 ;;;
 
 (define-module (ice-9 command-line)
-  #:autoload (system vm vm) (set-default-vm-engine! set-vm-engine! the-vm)
+  #:autoload (system vm vm) (set-default-vm-engine! set-vm-engine!)
   #:export (compile-shell-switches
             version-etc
             *GPLv3+*
@@ -40,15 +41,15 @@
             emit-bug-reporting-address))
 
 ;; An initial stab at i18n.
-(define _ gettext)
+(define G_ gettext)
 
 (define *GPLv3+*
-  (_ "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
+  (G_ "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law."))
 
 (define *LGPLv3+*
-  (_ "License LGPLv3+: GNU LGPL 3 or later <http://gnu.org/licenses/lgpl.html>.
+  (G_ "License LGPLv3+: GNU LGPL 3 or later <http://gnu.org/licenses/lgpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law."))
 
@@ -66,7 +67,7 @@ There is NO WARRANTY, to the extent permitted by law."))
 (define* (version-etc package version #:key
                       (port (current-output-port))
                       ;; FIXME: authors
-                      (copyright-year 2013)
+                      (copyright-year 2024)
                       (copyright-holder "Free Software Foundation, Inc.")
                       (copyright (format #f "Copyright (C) ~a ~a"
                                          copyright-year copyright-holder))
@@ -79,8 +80,8 @@ There is NO WARRANTY, to the extent permitted by law."))
 
   (if packager
       (if packager-version
-          (format port (_ "Packaged by ~a (~a)\n") packager packager-version)
-          (format port (_ "Packaged by ~a\n") packager)))
+          (format port (G_ "Packaged by ~a (~a)\n") packager packager-version)
+          (format port (G_ "Packaged by ~a\n") packager)))
   
   (display copyright port)
   (newline port)
@@ -98,15 +99,15 @@ There is NO WARRANTY, to the extent permitted by law."))
                                            package
                                            "/"))
                                      packager packager-bug-address)
-  (format port (_ "\nReport bugs to: ~a\n") bug-address)
+  (format port (G_ "\nReport bugs to: ~a\n") bug-address)
   (if (and packager packager-bug-address)
-      (format port (_ "Report ~a bugs to: ~a\n") packager packager-bug-address))
-  (format port (_ "~a home page: <~a>\n") package url)
+      (format port (G_ "Report ~a bugs to: ~a\n") packager packager-bug-address))
+  (format port (G_ "~a home page: <~a>\n") package url)
   (format port
-          (_ "General help using GNU software: <http://www.gnu.org/gethelp/>\n")))
+          (G_ "General help using GNU software: <http://www.gnu.org/gethelp/>\n")))
 
 (define *usage*
-  (_ "Evaluate code with Guile, interactively or from a script.
+  (G_ "Evaluate code with Guile, interactively or from a script.
 
   [-s] FILE      load source code from FILE, and exit
   -c EXPR        evalute expression EXPR, and exit
@@ -135,9 +136,14 @@ If FILE begins with `-' the -s switch is mandatory.
                  files.
   --listen[=P]   listen on a local port or a path for REPL clients;
                  if P is not given, the default is local port 37146
+  --statprof[=STYLE] profile using statprof and show results in STYLE
   -q             inhibit loading of user init file
   --use-srfi=LS  load SRFI modules for the SRFIs in LS,
                  which is a list of numbers like \"2,13,14\"
+  --r6rs         change initial Guile environment to better support
+                 R6RS
+  --r7rs         change initial Guile environment to better support
+                 R7RS
   -h, --help     display this help and exit
   -v, --version  display version information and exit
   \\              read arguments from following script lines"))
@@ -147,10 +153,11 @@ If FILE begins with `-' the -s switch is mandatory.
   (let ((port (if fatal?
                   (current-error-port)
                   (current-output-port))))
-    (if fmt
-        (apply format port fmt args))
+    (when fmt
+      (apply format port fmt args)
+      (newline port))
 
-    (format port (_ "Usage: ~a [OPTION]... [FILE]...\n") name)
+    (format port (G_ "Usage: ~a [OPTION]... [FILE]...\n") name)
     (display *usage* port)
     (newline port)
 
@@ -200,10 +207,12 @@ If FILE begins with `-' the -s switch is mandatory.
         (interactive? #t)
         (inhibit-user-init? #f)
         (turn-on-debugging? #f)
-        (turn-off-debugging? #f))
+        (turn-off-debugging? #f)
+        (statprof #f))
 
     (define (error fmt . args)
-      (apply shell-usage usage-name #t fmt args))
+      (apply shell-usage usage-name #t
+             (string-append "error: " fmt "~%") args))
 
     (define (parse args out)
       (cond
@@ -368,6 +377,14 @@ If FILE begins with `-' the -s switch is mandatory.
               (parse args
                      (cons `(use-srfis ',srfis) out))))
 
+           ((string=? "--r6rs" arg)
+            (parse args
+                   (cons '(install-r6rs!) out)))
+
+           ((string=? "--r7rs" arg)
+            (parse args
+                   (cons '(install-r7rs!) out)))
+
            ((string=? arg "--listen")   ; start a repl server
             (parse args
                    (cons '((@@ (system repl server) spawn-server)) out)))
@@ -391,6 +408,21 @@ If FILE begins with `-' the -s switch is mandatory.
                   (error "unknown argument to --listen"))))
               out)))
 
+           ((string=? arg "--statprof") ; run script with statprof
+            (set! statprof 'flat)
+            (parse args out))
+
+           ((string-prefix? "--statprof=" arg) ; run script with statprof
+            (let* ((style (substring arg 11)))
+              (set! statprof
+                    (cond
+                     ((string=? style "flat") 'flat)
+                     ((string=? style "anomalies") 'anomalies)
+                     ((string=? style "tree") 'tree)
+                     (else
+                      (error "unknown argument to --statprof")))))
+            (parse args out))
+
            ((or (string=? arg "-h") (string=? arg "--help"))
             (shell-usage usage-name #f)
             (exit 0))
@@ -405,7 +437,7 @@ If FILE begins with `-' the -s switch is mandatory.
             (exit 0))
 
            (else
-            (error "Unrecognized switch ~a" arg)))))))
+            (error "unrecognized switch ~a" arg)))))))
 
     (define (finish args out)
       ;; Check to make sure the -ds got a -s.
@@ -422,7 +454,7 @@ If FILE begins with `-' the -s switch is mandatory.
               (and interactive? (not turn-off-debugging?)))
           (begin
             (set-default-vm-engine! 'debug)
-            (set-vm-engine! (the-vm) 'debug)))
+            (set-vm-engine! 'debug)))
       
       ;; Return this value.
       `(;; It would be nice not to load up (ice-9 control), but the
@@ -450,6 +482,11 @@ If FILE begins with `-' the -s switch is mandatory.
                           (cons ,path %load-compiled-path)))
                  user-load-compiled-path)
 
+          ;; If statprof was requested, start it.
+          ,@(if statprof
+                '(((@ (statprof) statprof-start)))
+                '())
+
           ;; Put accumulated actions in their correct order.
           ,@(reverse! out)
 
@@ -457,6 +494,15 @@ If FILE begins with `-' the -s switch is mandatory.
           ,@(if entry-point
                 `((,entry-point (command-line)))
                 '())
+
+          ;; After doing all actions, stop statprof and print
+          ;; results. Avoid continuing the profiling inside the REPL,
+          ;; since it has ,profile.
+          ,@(if statprof
+                `(((@ (statprof) statprof-stop))
+                  ((@ (statprof) statprof-display) #:style ',statprof))
+                '())
+
           ,(if interactive?
                ;; If we didn't end with a -c or a -s, start the
                ;; repl.
