@@ -20,7 +20,7 @@
 
 ;;; Commentary:
 ;;
-;;Routines to generare @code{stexi} documentation for objects and
+;;Routines to generate @code{stexi} documentation for objects and
 ;;modules.
 ;;
 ;;Note that in this context, an @dfn{object} is just a value associated
@@ -33,10 +33,8 @@
   #:use-module (oop goops)
   #:use-module (texinfo)
   #:use-module (texinfo plain-text)
-  #:use-module (srfi srfi-13)
   #:use-module (ice-9 session)
   #:use-module (ice-9 documentation)
-  #:use-module (ice-9 optargs)
   #:use-module ((sxml transform) #:select (pre-post-order))
   #:export (module-stexi-documentation
             script-stexi-documentation
@@ -288,16 +286,11 @@
           (else (lp (cdr forms))))))
 
 (define* (module-stexi-documentation sym-name
-                                     #:optional %docs-resolver
                                      #:key (docs-resolver
-                                            (or %docs-resolver
-                                                (lambda (name def) def))))
+                                            (lambda (name def) def)))
   "Return documentation for the module named @var{sym-name}. The
 documentation will be formatted as @code{stexi}
  (@pxref{texinfo,texinfo})."
-  (if %docs-resolver
-      (issue-deprecation-warning
-       "module-stexi-documentation: use #:docs-resolver instead of a positional argument."))
   (let* ((commentary (and=> (module-commentary sym-name)
                             (lambda (x) (string-trim-both x #\newline))))
          (stexi (string->stexi commentary))
